/* *****************************************************************************
 *
 * Filename: CalendarBaseCustomizer.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  21 December 2002, 13:53
 *
 * Purpose:  Customizer for all Calendar beans (Table, List, etc.).
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.awt.*;
import java.beans.*;
import java.util.ResourceBundle;
import javax.swing.*;

/**
 *
 * @author  Robert
 */
public class CalendarBaseCustomizer
    extends javax.swing.JPanel
    implements java.beans.Customizer
{
    // Statics
    private static final ResourceBundle resources = ResourceBundle.getBundle( "com/tsac/calmaker/beans/CalendarBeans" );
    private static final int[] sizes = { 8, 10, 12, 14, 16, 18, 20, 24, 28, 32, 36 };
    private static final String[] styles = {
        resources.getString("Plain"),  resources.getString("Bold"),
        resources.getString("Italic"), resources.getString("Bold_Italic")
    };

    // State
    private CalendarBase _object;
    
    // Cached
    private Image _cellBorderColorIcon;
    private Image _cellTextColorIcon;
    private Image _cellWendTextColorIcon;
    private PropertyChangeSupport _pcsupport = null;
    
    /** Creates new form CalendarBaseCustomizer */
    public CalendarBaseCustomizer()
    {
        initComponents();
        
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        String[] fonts = ge.getAvailableFontFamilyNames();
        for( int n = 0; n < fonts.length; n ++ )
        {
            dayFontFamilyCmb.addItem( fonts[n] );
            dateFontFamilyCmb.addItem( fonts[n] );
            diaryFontFamilyCmb.addItem( fonts[n] );
        }
        
        for( int n = 0; n < styles.length; n++ )
        {
            dayFontStyleCmb.addItem( styles[n] );
            dateFontStyleCmb.addItem( styles[n] );
            diaryFontStyleCmb.addItem( styles[n] );
        }
        
        for( int n = 0; n < sizes.length; n++ )
        {
            dayFontSizeCmb.addItem( Integer.toString( sizes[n] ) );
            dateFontSizeCmb.addItem( Integer.toString( sizes[n] ) );
            diaryFontSizeCmb.addItem( Integer.toString( sizes[n] ) );
        }
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents()//GEN-BEGIN:initComponents
    {
        java.awt.GridBagConstraints gridBagConstraints;

        colorPnl = new javax.swing.JPanel();
        cellBorderColorLbl = new javax.swing.JLabel();
        cellBorderColorPnl = new javax.swing.JPanel();
        cellBorderColor = new javax.swing.JLabel();
        cellBorderColorBtn = new javax.swing.JButton();
        cellTextColorLbl = new javax.swing.JLabel();
        cellTextColorPnl = new javax.swing.JPanel();
        cellTextColor = new javax.swing.JLabel();
        cellTextColorBtn = new javax.swing.JButton();
        cellWendTextColorLbl = new javax.swing.JLabel();
        cellWendTextColorPnl = new javax.swing.JPanel();
        cellWendTextColor = new javax.swing.JLabel();
        cellWendTextBtn = new javax.swing.JButton();
        fontPnl = new javax.swing.JPanel();
        dayFontLbl = new javax.swing.JLabel();
        dayFontPnl = new javax.swing.JPanel();
        dayFontFamilyCmb = new javax.swing.JComboBox();
        dayFontStyleCmb = new javax.swing.JComboBox();
        dayFontSizeCmb = new javax.swing.JComboBox();
        dateFontLbl = new javax.swing.JLabel();
        dateFontPnl = new javax.swing.JPanel();
        dateFontFamilyCmb = new javax.swing.JComboBox();
        dateFontStyleCmb = new javax.swing.JComboBox();
        dateFontSizeCmb = new javax.swing.JComboBox();
        diaryFontLbl = new javax.swing.JLabel();
        diaryFontPnl = new javax.swing.JPanel();
        diaryFontFamilyCmb = new javax.swing.JComboBox();
        diaryFontStyleCmb = new javax.swing.JComboBox();
        diaryFontSizeCmb = new javax.swing.JComboBox();

        setLayout(new java.awt.GridBagLayout());

        colorPnl.setLayout(new java.awt.GridBagLayout());

        colorPnl.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), java.util.ResourceBundle.getBundle("com/tsac/calmaker/beans/CalendarBeans").getString("Colors")));
        cellBorderColorLbl.setText(resources.getString("Cell_Border_"));
        cellBorderColorLbl.setDisplayedMnemonic(resources.getString( "Cell_Border_Mnemonic" ).charAt(0));
        cellBorderColorLbl.setLabelFor(cellBorderColorBtn);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 2, 2);
        colorPnl.add(cellBorderColorLbl, gridBagConstraints);

        cellBorderColorPnl.setLayout(new java.awt.GridBagLayout());

        cellBorderColorPnl.setBorder(new javax.swing.border.EtchedBorder());
        cellBorderColor.setText("[255,255,255]");
        cellBorderColor.setToolTipText(resources.getString("Cell_Border_Tooltip"));
        cellBorderColor.setMinimumSize(new java.awt.Dimension(110, 25));
        cellBorderColor.setPreferredSize(new java.awt.Dimension(110, 25));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 4, 0, 2);
        cellBorderColorPnl.add(cellBorderColor, gridBagConstraints);

        cellBorderColorBtn.setText("...");
        cellBorderColorBtn.setToolTipText(resources.getString("Color_Button_Tooltip"));
        cellBorderColorBtn.setPreferredSize(new java.awt.Dimension(24, 24));
        cellBorderColorBtn.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                cellBorderColorBtnActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 2, 0, 0);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        cellBorderColorPnl.add(cellBorderColorBtn, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(4, 2, 2, 4);
        colorPnl.add(cellBorderColorPnl, gridBagConstraints);

        cellTextColorLbl.setText(java.util.ResourceBundle.getBundle("com/tsac/calmaker/beans/CalendarBeans").getString("Cell_Text_"));
        cellTextColorLbl.setDisplayedMnemonic(resources.getString( "Cell_Text_Mnemonic" ).charAt(0));
        cellTextColorLbl.setLabelFor(cellTextColorBtn);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 2);
        colorPnl.add(cellTextColorLbl, gridBagConstraints);

        cellTextColorPnl.setLayout(new java.awt.GridBagLayout());

        cellTextColorPnl.setBorder(new javax.swing.border.EtchedBorder());
        cellTextColor.setText("[255,255,255]");
        cellTextColor.setToolTipText(resources.getString("Cell_Text_Tooltip"));
        cellTextColor.setMinimumSize(new java.awt.Dimension(110, 25));
        cellTextColor.setPreferredSize(new java.awt.Dimension(110, 25));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 4, 0, 2);
        cellTextColorPnl.add(cellTextColor, gridBagConstraints);

        cellTextColorBtn.setText("...");
        cellTextColorBtn.setToolTipText(resources.getString("Color_Button_Tooltip"));
        cellTextColorBtn.setPreferredSize(new java.awt.Dimension(24, 24));
        cellTextColorBtn.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                cellTextColorBtnActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 2, 0, 0);
        cellTextColorPnl.add(cellTextColorBtn, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 4);
        colorPnl.add(cellTextColorPnl, gridBagConstraints);

        cellWendTextColorLbl.setText(java.util.ResourceBundle.getBundle("com/tsac/calmaker/beans/CalendarBeans").getString("Cell_Text_Wend_"));
        cellWendTextColorLbl.setDisplayedMnemonic(resources.getString( "Cell_Text_Wend_Mnemonic" ).charAt(0));
        cellWendTextColorLbl.setLabelFor(cellWendTextBtn);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 2);
        colorPnl.add(cellWendTextColorLbl, gridBagConstraints);

        cellWendTextColorPnl.setLayout(new java.awt.GridBagLayout());

        cellWendTextColorPnl.setBorder(new javax.swing.border.EtchedBorder());
        cellWendTextColor.setText("[255,255,255]");
        cellWendTextColor.setToolTipText(resources.getString("Cell_Text_Wend_Tooltip"));
        cellWendTextColor.setMinimumSize(new java.awt.Dimension(110, 25));
        cellWendTextColor.setPreferredSize(new java.awt.Dimension(110, 25));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 4, 0, 2);
        cellWendTextColorPnl.add(cellWendTextColor, gridBagConstraints);

        cellWendTextBtn.setText("...");
        cellWendTextBtn.setToolTipText(resources.getString("Color_Button_Tooltip"));
        cellWendTextBtn.setPreferredSize(new java.awt.Dimension(24, 24));
        cellWendTextBtn.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                cellWendTextBtnActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 2, 0, 0);
        cellWendTextColorPnl.add(cellWendTextBtn, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 4, 4);
        colorPnl.add(cellWendTextColorPnl, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 0, 4);
        add(colorPnl, gridBagConstraints);

        fontPnl.setLayout(new java.awt.GridBagLayout());

        fontPnl.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), java.util.ResourceBundle.getBundle("com/tsac/calmaker/beans/CalendarBeans").getString("Fonts")));
        dayFontLbl.setText(java.util.ResourceBundle.getBundle("com/tsac/calmaker/beans/CalendarBeans").getString("Day_"));
        dayFontLbl.setDisplayedMnemonic(resources.getString( "Day_Mnemonic" ).charAt(0));
        dayFontLbl.setLabelFor(dayFontFamilyCmb);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        fontPnl.add(dayFontLbl, gridBagConstraints);

        dayFontPnl.setLayout(new java.awt.GridBagLayout());

        dayFontFamilyCmb.setToolTipText(resources.getString("Day_Family_Tooltip"));
        dayFontFamilyCmb.setPreferredSize(new java.awt.Dimension(150, 25));
        dayFontFamilyCmb.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                dayFontFamilyCmbActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 1);
        dayFontPnl.add(dayFontFamilyCmb, gridBagConstraints);

        dayFontStyleCmb.setToolTipText(resources.getString("Day_Style_Tooltip"));
        dayFontStyleCmb.setPreferredSize(new java.awt.Dimension(75, 25));
        dayFontStyleCmb.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                dayFontStyleCmbActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 1, 0, 1);
        dayFontPnl.add(dayFontStyleCmb, gridBagConstraints);

        dayFontSizeCmb.setEditable(true);
        dayFontSizeCmb.setToolTipText(resources.getString("Day_Size_Tooltip"));
        dayFontSizeCmb.setPreferredSize(new java.awt.Dimension(50, 25));
        dayFontSizeCmb.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                dayFontSizeCmbActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 1, 0, 0);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        dayFontPnl.add(dayFontSizeCmb, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 4);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        fontPnl.add(dayFontPnl, gridBagConstraints);

        dateFontLbl.setText(java.util.ResourceBundle.getBundle("com/tsac/calmaker/beans/CalendarBeans").getString("Date_"));
        dateFontLbl.setDisplayedMnemonic(resources.getString( "Date_Mnemonic" ).charAt(0));
        dateFontLbl.setLabelFor(dateFontFamilyCmb);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        fontPnl.add(dateFontLbl, gridBagConstraints);

        dateFontPnl.setLayout(new java.awt.GridBagLayout());

        dateFontFamilyCmb.setToolTipText(resources.getString("Date_Family_Tooltip"));
        dateFontFamilyCmb.setPreferredSize(new java.awt.Dimension(150, 25));
        dateFontFamilyCmb.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                dateFontFamilyCmbActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 1);
        dateFontPnl.add(dateFontFamilyCmb, gridBagConstraints);

        dateFontStyleCmb.setToolTipText(resources.getString("Date_Style_Tooltip"));
        dateFontStyleCmb.setPreferredSize(new java.awt.Dimension(75, 25));
        dateFontStyleCmb.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                dateFontStyleCmbActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 1, 0, 1);
        dateFontPnl.add(dateFontStyleCmb, gridBagConstraints);

        dateFontSizeCmb.setEditable(true);
        dateFontSizeCmb.setToolTipText(resources.getString("Date_Size_Tooltip"));
        dateFontSizeCmb.setPreferredSize(new java.awt.Dimension(50, 25));
        dateFontSizeCmb.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                dateFontSizeCmbActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 1, 0, 0);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        dateFontPnl.add(dateFontSizeCmb, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 4);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        fontPnl.add(dateFontPnl, gridBagConstraints);

        diaryFontLbl.setText(java.util.ResourceBundle.getBundle("com/tsac/calmaker/beans/CalendarBeans").getString("Diary_"));
        diaryFontLbl.setDisplayedMnemonic(resources.getString( "Diary_Mnemonic" ).charAt(0));
        diaryFontLbl.setLabelFor(diaryFontFamilyCmb);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        fontPnl.add(diaryFontLbl, gridBagConstraints);

        diaryFontPnl.setLayout(new java.awt.GridBagLayout());

        diaryFontFamilyCmb.setToolTipText(resources.getString("Diary_Family_Tooltip"));
        diaryFontFamilyCmb.setPreferredSize(new java.awt.Dimension(150, 25));
        diaryFontFamilyCmb.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                diaryFontFamilyCmbActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 1);
        diaryFontPnl.add(diaryFontFamilyCmb, gridBagConstraints);

        diaryFontStyleCmb.setToolTipText(resources.getString("Diary_Style_Tooltip"));
        diaryFontStyleCmb.setPreferredSize(new java.awt.Dimension(75, 25));
        diaryFontStyleCmb.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                diaryFontStyleCmbActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 1, 0, 1);
        diaryFontPnl.add(diaryFontStyleCmb, gridBagConstraints);

        diaryFontSizeCmb.setEditable(true);
        diaryFontSizeCmb.setToolTipText(resources.getString("Diary_Size_Tooltip"));
        diaryFontSizeCmb.setPreferredSize(new java.awt.Dimension(50, 25));
        diaryFontSizeCmb.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                diaryFontSizeCmbActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 1, 0, 0);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        diaryFontPnl.add(diaryFontSizeCmb, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 4, 4);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        fontPnl.add(diaryFontPnl, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 4, 4, 4);
        add(fontPnl, gridBagConstraints);

    }//GEN-END:initComponents

    private void diaryFontSizeCmbActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_diaryFontSizeCmbActionPerformed
    {//GEN-HEADEREND:event_diaryFontSizeCmbActionPerformed
        actionDiaryFontChanged();
    }//GEN-LAST:event_diaryFontSizeCmbActionPerformed

    private void diaryFontStyleCmbActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_diaryFontStyleCmbActionPerformed
    {//GEN-HEADEREND:event_diaryFontStyleCmbActionPerformed
        actionDiaryFontChanged();
    }//GEN-LAST:event_diaryFontStyleCmbActionPerformed

    private void diaryFontFamilyCmbActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_diaryFontFamilyCmbActionPerformed
    {//GEN-HEADEREND:event_diaryFontFamilyCmbActionPerformed
        actionDiaryFontChanged();
    }//GEN-LAST:event_diaryFontFamilyCmbActionPerformed

    private void dateFontSizeCmbActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_dateFontSizeCmbActionPerformed
    {//GEN-HEADEREND:event_dateFontSizeCmbActionPerformed
        actionDateFontChanged();
    }//GEN-LAST:event_dateFontSizeCmbActionPerformed

    private void dateFontStyleCmbActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_dateFontStyleCmbActionPerformed
    {//GEN-HEADEREND:event_dateFontStyleCmbActionPerformed
        actionDateFontChanged();
    }//GEN-LAST:event_dateFontStyleCmbActionPerformed

    private void dateFontFamilyCmbActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_dateFontFamilyCmbActionPerformed
    {//GEN-HEADEREND:event_dateFontFamilyCmbActionPerformed
        actionDateFontChanged();
    }//GEN-LAST:event_dateFontFamilyCmbActionPerformed

    private void dayFontSizeCmbActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_dayFontSizeCmbActionPerformed
    {//GEN-HEADEREND:event_dayFontSizeCmbActionPerformed
        actionDayFontChanged();
    }//GEN-LAST:event_dayFontSizeCmbActionPerformed

    private void dayFontStyleCmbActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_dayFontStyleCmbActionPerformed
    {//GEN-HEADEREND:event_dayFontStyleCmbActionPerformed
        actionDayFontChanged();
    }//GEN-LAST:event_dayFontStyleCmbActionPerformed

    private void dayFontFamilyCmbActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_dayFontFamilyCmbActionPerformed
    {//GEN-HEADEREND:event_dayFontFamilyCmbActionPerformed
        actionDayFontChanged();
    }//GEN-LAST:event_dayFontFamilyCmbActionPerformed

    private void cellWendTextBtnActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cellWendTextBtnActionPerformed
    {//GEN-HEADEREND:event_cellWendTextBtnActionPerformed
        JColorChooser colordlg = new JColorChooser();
        Color newColor = colordlg.showDialog( this, "Color Chooser", _object.getCellWeekendTextColor() );
        
        if( newColor != null )
        {
            Color oldColor = _object.getCellWeekendTextColor();
            
            updateCellWendTextColor( newColor );
            _object.setCellWeekendTextColor( newColor );
            
            if( _pcsupport != null )
                _pcsupport.firePropertyChange( "cellWeekendTextColor", oldColor, newColor );            
        }
    }//GEN-LAST:event_cellWendTextBtnActionPerformed

    private void cellTextColorBtnActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cellTextColorBtnActionPerformed
    {//GEN-HEADEREND:event_cellTextColorBtnActionPerformed
        JColorChooser colordlg = new JColorChooser();
        Color newColor = colordlg.showDialog( this, "Color Chooser", _object.getCellTextColor() );
        
        if( newColor != null )
        {
            Color oldColor = _object.getCellTextColor();
            
            updateCellTextColor( newColor );
            _object.setCellTextColor( newColor );
            
            if( _pcsupport != null )
                _pcsupport.firePropertyChange( "cellTextColor", oldColor, newColor );            
        }
    }//GEN-LAST:event_cellTextColorBtnActionPerformed

    private void cellBorderColorBtnActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cellBorderColorBtnActionPerformed
    {//GEN-HEADEREND:event_cellBorderColorBtnActionPerformed
        JColorChooser colordlg = new JColorChooser();
        Color newColor = colordlg.showDialog( this, "Color Chooser", _object.getCellBorderColor() );
        
        if( newColor != null )
        {
            Color oldColor = _object.getCellBorderColor();
            
            updateCellBorderColor( newColor );
            _object.setCellBorderColor( newColor );
            
            if( _pcsupport != null )
                _pcsupport.firePropertyChange( "cellBorderColor", oldColor, newColor );            
        }
    }//GEN-LAST:event_cellBorderColorBtnActionPerformed

    public void setObject(Object obj)
    {
        if( obj instanceof CalendarBase )
        {
            CalendarBase cb = (CalendarBase) obj;
            _object = cb;
            _pcsupport = new PropertyChangeSupport( cb );

            updateCellBorderColor( cb.getCellBorderColor() );
            updateCellTextColor( cb.getCellTextColor() );
            updateCellWendTextColor( cb.getCellWeekendTextColor() );
            updateFont( dayFontFamilyCmb, dayFontStyleCmb, dayFontSizeCmb, cb.getDayFont() );
            updateFont( dateFontFamilyCmb, dateFontStyleCmb, dateFontSizeCmb, cb.getDateFont() );
            updateFont( diaryFontFamilyCmb, diaryFontStyleCmb, diaryFontSizeCmb, cb.getDiaryFont() );
        }
    }    
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel cellWendTextColorPnl;
    private javax.swing.JLabel cellTextColor;
    private javax.swing.JComboBox dayFontStyleCmb;
    private javax.swing.JComboBox dayFontSizeCmb;
    private javax.swing.JLabel dayFontLbl;
    private javax.swing.JComboBox dayFontFamilyCmb;
    private javax.swing.JComboBox diaryFontSizeCmb;
    private javax.swing.JLabel diaryFontLbl;
    private javax.swing.JPanel dayFontPnl;
    private javax.swing.JComboBox diaryFontFamilyCmb;
    private javax.swing.JLabel cellTextColorLbl;
    private javax.swing.JLabel cellBorderColorLbl;
    private javax.swing.JPanel fontPnl;
    private javax.swing.JPanel diaryFontPnl;
    private javax.swing.JComboBox dateFontSizeCmb;
    private javax.swing.JPanel cellTextColorPnl;
    private javax.swing.JComboBox diaryFontStyleCmb;
    private javax.swing.JLabel dateFontLbl;
    private javax.swing.JPanel cellBorderColorPnl;
    private javax.swing.JComboBox dateFontFamilyCmb;
    private javax.swing.JLabel cellBorderColor;
    private javax.swing.JPanel dateFontPnl;
    private javax.swing.JLabel cellWendTextColor;
    private javax.swing.JPanel colorPnl;
    private javax.swing.JButton cellWendTextBtn;
    private javax.swing.JButton cellTextColorBtn;
    private javax.swing.JButton cellBorderColorBtn;
    private javax.swing.JLabel cellWendTextColorLbl;
    private javax.swing.JComboBox dateFontStyleCmb;
    // End of variables declaration//GEN-END:variables

    
    public void addPropertyChangeListener( PropertyChangeListener l )
    {
        if( _pcsupport != null )
            _pcsupport.addPropertyChangeListener( l );
    }

    
    public void removePropertyChangeListener( PropertyChangeListener l )
    {
        if( _pcsupport != null )
            _pcsupport.removePropertyChangeListener( l );
    }
    
    
    protected void updateCellBorderColor( Color c )
    {
        // Make sure the icon exists
        if( _cellBorderColorIcon == null )
        {
            _cellBorderColorIcon = createImage( 16, 16 );
            
            if( _cellBorderColorIcon != null )
                cellBorderColor.setIcon( new ImageIcon( _cellBorderColorIcon ) );
        }

        // Update the icon
        if( _cellBorderColorIcon != null )
        {
            Graphics g = _cellBorderColorIcon.getGraphics();
            g.setColor( c );
            g.fillRect( 0, 0, _cellBorderColorIcon.getWidth(null), _cellBorderColorIcon.getHeight(null) );
            g.setColor( Color.black );
            g.drawRect( 0, 0, _cellBorderColorIcon.getWidth(null) - 1, _cellBorderColorIcon.getHeight(null) - 1 );
        }
        
        // Update the text
        cellBorderColor.setText( "[" + c.getRed() + "," + c.getGreen() + "," + c.getBlue() + "]" );
        
        // Force a repaint
        cellBorderColor.repaint();
    }

    
    protected void updateCellTextColor( Color c )
    {
        // Make sure the icon exists
        if( _cellTextColorIcon == null )
        {
            _cellTextColorIcon = createImage( 16, 16 );
            
            if( _cellTextColorIcon != null )
                cellTextColor.setIcon( new ImageIcon( _cellTextColorIcon ) );
        }

        // Update the icon
        if( _cellTextColorIcon != null )
        {
            Graphics g = _cellTextColorIcon.getGraphics();
            g.setColor( c );
            g.fillRect( 0, 0, _cellTextColorIcon.getWidth(null), _cellTextColorIcon.getHeight(null) );
            g.setColor( Color.black );
            g.drawRect( 0, 0, _cellTextColorIcon.getWidth(null) - 1, _cellTextColorIcon.getHeight(null) - 1 );
        }
        
        // Update the text
        cellTextColor.setText( "[" + c.getRed() + "," + c.getGreen() + "," + c.getBlue() + "]" );
        
        // Force a repaint
        cellTextColor.repaint();
    }

    
    protected void updateCellWendTextColor( Color c )
    {
        // Make sure the icon exists
        if( _cellWendTextColorIcon == null )
        {
            _cellWendTextColorIcon = createImage( 16, 16 );
            
            if( _cellWendTextColorIcon != null )
                cellWendTextColor.setIcon( new ImageIcon( _cellWendTextColorIcon ) );
        }

        // Update the icon
        if( _cellTextColorIcon != null )
        {
            Graphics g = _cellWendTextColorIcon.getGraphics();
            g.setColor( c );
            g.fillRect( 0, 0, _cellWendTextColorIcon.getWidth(null), _cellWendTextColorIcon.getHeight(null) );
            g.setColor( Color.black );
            g.drawRect( 0, 0, _cellWendTextColorIcon.getWidth(null) - 1, _cellWendTextColorIcon.getHeight(null) - 1 );
        }
        
        // Update the text
        cellWendTextColor.setText( "[" + c.getRed() + "," + c.getGreen() + "," + c.getBlue() + "]" );
        
        // Force a repaint
        cellWendTextColor.repaint();
    }

    
    protected void updateFont( JComboBox familyCmb, JComboBox styleCmb, JComboBox sizeCmb, Font f )
    {
        // Save object and turn prevent selection events writing back to the object
        CalendarBase obj = _object;
        _object = null;
        
        familyCmb.setSelectedItem( f.getFamily() );
        styleCmb.setSelectedIndex( f.getStyle() );
        sizeCmb.setSelectedItem( Integer.toString(f.getSize()) );
        
        // Restore the object
        _object = obj;
    }
    
    
    protected void actionDayFontChanged()
    {
        // Make sure object has been set
        if( _object == null )
            return;
        
        try
        {
            Font newFont = new Font( (String) dayFontFamilyCmb.getSelectedItem(), dayFontStyleCmb.getSelectedIndex(), Integer.parseInt( (String) dayFontSizeCmb.getSelectedItem() ) );
            Font oldFont = _object.getDayFont();
            
            _object.setDayFont( newFont );
            
            if( _pcsupport != null )
                _pcsupport.firePropertyChange( "dayFont", oldFont, newFont );
        }
        catch( NumberFormatException e )
        {
            dayFontSizeCmb.setSelectedItem( Integer.toString(_object.getDayFont().getSize()) );
        }
    }
    
    
    protected void actionDateFontChanged()
    {
        // Make sure object has been set
        if( _object == null )
            return;
        
        try
        {
            Font newFont = new Font( (String) dateFontFamilyCmb.getSelectedItem(), dateFontStyleCmb.getSelectedIndex(), Integer.parseInt( (String) dateFontSizeCmb.getSelectedItem() ) );
            Font oldFont = _object.getDateFont();
            
            _object.setDateFont( newFont );
            
            if( _pcsupport != null )
                _pcsupport.firePropertyChange( "dateFont", oldFont, newFont );
        }
        catch( NumberFormatException e )
        {
            dateFontSizeCmb.setSelectedItem( Integer.toString(_object.getDateFont().getSize()) );
        }
    }
    
    
    protected void actionDiaryFontChanged()
    {
        // Make sure object has been set
        if( _object == null )
            return;
        
        try
        {
            Font newFont = new Font( (String) diaryFontFamilyCmb.getSelectedItem(), diaryFontStyleCmb.getSelectedIndex(), Integer.parseInt( (String) diaryFontSizeCmb.getSelectedItem() ) );
            Font oldFont = _object.getDiaryFont();
            
            _object.setDiaryFont( newFont );
            
            if( _pcsupport != null )
                _pcsupport.firePropertyChange( "diaryFont", oldFont, newFont );
        }
        catch( NumberFormatException e )
        {
            diaryFontSizeCmb.setSelectedItem( Integer.toString(_object.getDiaryFont().getSize()) );
        }
    }
}
