/* *****************************************************************************
 *
 * Filename: MonthImageCustomizer.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  19 December 2002, 11:49
 *
 * Purpose:  Customizer for MonthImage bean.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.awt.Dimension;
import java.beans.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.text.SimpleDateFormat;
import java.util.ResourceBundle;

/**
 *
 * @author  Robert
 */
public class MonthImageCustomizer
    extends javax.swing.JPanel
    implements java.beans.Customizer, java.beans.PropertyChangeListener
{
    // Statics
    private static final ResourceBundle resources = ResourceBundle.getBundle( "com/tsac/calmaker/beans/CalendarBeans" );
    private static final int MONTH_COLUMN_INDEX = 0;
    private static final int IMAGE_COLUMN_INDEX = 1;
    
    // State
    private MonthImage _object;
    private TableModel _model;
    
    // Cached
    private PropertyChangeSupport _pcsupport = null;
   
    
    private class MonthImageTableModel
        extends AbstractTableModel
    {
        private Class[] types = new Class[] {
            java.lang.String.class, java.lang.Object.class
        };

        private boolean[] canEdit = new boolean[] {
            false, true
        };

        private String[] columnNames = new String[] {
            resources.getString("Month"), resources.getString("Image")
        };

        private Object[][] _data;
        
        public MonthImageTableModel( Object[][] data ) {
            _data = data;
        }

        public Class getColumnClass( int columnIndex ) {
            return types[columnIndex];
        }
        
        public String getColumnName( int columnIndex ) {
            return columnNames[columnIndex];
        }

        public boolean isCellEditable( int rowIndex, int columnIndex ) {
            return canEdit[columnIndex];
        }
        
        public int getColumnCount() {
            return columnNames.length;
        }        
        
        public int getRowCount() {
            return _data.length;
        }
        
        public Object getValueAt( int row, int col ) {
             return _data[row][col];
        }
                
        public void setValueAt( Object value, int row, int col ) {
           _data[row][col] = value;
            fireTableCellUpdated( row, col );
        }
    }
    
    
    /** Creates new customizer MonthImageCustomizer */
    public MonthImageCustomizer()
    {
        initComponents();
        
        // Install our custom editor for all of column 1
        imageTbl.getColumnModel().getColumn( IMAGE_COLUMN_INDEX ).setCellEditor( new MonthImageCellEditor() );
        
        // Update the object and fire a property change when table updated
        _model.addTableModelListener( new TableModelListener() {
            public void tableChanged( TableModelEvent e ) {
                if( (e.getFirstRow() == e.getLastRow()) && (e.getFirstRow() >= 0) )
                {
                    String[] temp = _object.getImages();
                    String[] oldImages = new String[temp.length];
                    System.arraycopy( temp, 0, oldImages, 0, temp.length );
                    _object.setImages( e.getFirstRow(), (String) _model.getValueAt( e.getFirstRow(), IMAGE_COLUMN_INDEX ) );
                    String[] newImages = _object.getImages();

                    if( _pcsupport != null )
                        _pcsupport.firePropertyChange( "images", oldImages, newImages );
                }
            }
        } );
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents()//GEN-BEGIN:initComponents
    {
        java.awt.GridBagConstraints gridBagConstraints;

        scrollpane = new javax.swing.JScrollPane();
        imageTbl = new javax.swing.JTable();

        setLayout(new java.awt.GridBagLayout());

        setPreferredSize(new java.awt.Dimension(342, 250));
        scrollpane.setPreferredSize(new java.awt.Dimension(326, 250));
        imageTbl.setModel(_model = createTableModel());
        scrollpane.setViewportView(imageTbl);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.ipady = 250;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
        add(scrollpane, gridBagConstraints);

    }//GEN-END:initComponents

    public void setObject(Object obj)
    {
        if( obj instanceof MonthImage )
        {
            unregisterListeners();      // Unregister from previous object

            MonthImage mi = (MonthImage) obj;
            _object = mi;
            _pcsupport = new PropertyChangeSupport( mi );
            readImages();
            registerListeners();        // Register with new object
        }
    }    
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane scrollpane;
    private javax.swing.JTable imageTbl;
    // End of variables declaration//GEN-END:variables
    

    public void addPropertyChangeListener( PropertyChangeListener l )
    {
        if( _pcsupport != null )
            _pcsupport.addPropertyChangeListener( l );
    }

    
    public void removePropertyChangeListener( PropertyChangeListener l )
    {
        if( _pcsupport != null )
            _pcsupport.removePropertyChangeListener( l );
    }

    
    private TableModel createTableModel()
    {
        // Populate Table with month names
        String[] months = new SimpleDateFormat().getDateFormatSymbols().getMonths();
        Object[][] modelData = new Object[months.length-1][];
        for( int n = 0; n < months.length-1; n++ )
            modelData[n] = new Object[] { months[n], null };
                    
        TableModel model = new MonthImageTableModel( modelData );
        
        return model;
    }
    
    
    private void registerListeners()
    {
        // Listen for changes in the image list of our MonthImage
        _object.addPropertyChangeListener( MonthImage.IMAGES_PROPERTY, this );
    }
    
    
    private void unregisterListeners()
    {
        if( _object != null )
        {
            _object.removePropertyChangeListener( MonthImage.IMAGES_PROPERTY, this );
        }
    }
    
    
    // Implements PropertyChangeListener
    public void propertyChange( PropertyChangeEvent e )
    {
        if( MonthImage.IMAGES_PROPERTY.equals( e.getPropertyName() ) )
            readImages();
    }
    
    
    private void readImages()
    {
        String[] images = _object.getImages();

        for( int n = 0; n < images.length; n++ )
            _model.setValueAt( images[n], n, IMAGE_COLUMN_INDEX );
    }
}
