/* *****************************************************************************
 *
 * Filename: TableCalendar.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  17 December 2002, 11:33
 *
 * Purpose:  Calendar bean with a table-like form.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.beans.*;
import java.awt.*;
import java.awt.geom.Rectangle2D;

import com.tsac.calmaker.*;

public class TableCalendar
    extends CalendarBase
{
	public TableCalendar()
	{
        setPreferredSize( new Dimension( 200, 160 ) ); 
	}
    
    
    private int _longestdate;   // From font

    
    protected void calclayout()
    {
        _resized = false;
        
        int width = this.getWidth();
        int height = this.getHeight();

        _cellwidth = (width - 1 - _inset) / 7;
        _leftmargin = (width - _cellwidth * 7) / 2;
            
        
        // reduce font size to fit width of a column (day names)
        String longestdaystr = " " + Constants.getLongestDay( this, _dayfont ) + " ";
        _actualdayfont = Constants.reduceFontWidthToFit( this, _dayfont, longestdaystr, _cellwidth );

        // if headings will be less than 8pt then resort to short names
        if( _actualdayfont.getSize() < 8 )
        {
            _actualdayfont = Constants.reduceFontWidthToFit(
                this, new Font( _dayfont.getFamily(), _dayfont.getStyle(), 12 ), " We ", _cellwidth );
            _useshortdaynames = true;
        }
        else
            _useshortdaynames = false;
            
        _dayfontmetrics = getFontMetrics( _actualdayfont );
                        
        _cellheight = (height - _dayfontmetrics.getHeight() - _botmargin - 1) / 6;

        // reduce font size to fit width of a column (dates)
        _actualdatefont = Constants.reduceFontWidthToFit( this, _datefont, "00", _cellwidth );
        _datefontmetrics = getFontMetrics( _actualdatefont );
        _longestdate = _datefontmetrics.stringWidth( "00 " );
        
        _actualdiaryfont = _diaryfont;
        _diaryfontmetrics = getFontMetrics( _actualdiaryfont );
    }

    
    public void paint( Graphics og, double scale )
    {
        Graphics2D g = (Graphics2D) og;
        
        if( _resized  )
            calclayout();
                
        float width      = (float) (scale * this.getWidth());
        float height     = (float) (scale * this.getHeight());
        float cellwidth  = (float) (scale * _cellwidth);
        float cellheight = (float) (scale * _cellheight);
        float leftmargin = (float) (scale * _leftmargin);
        float longestdate = (float) (scale * _longestdate);
        
        Font dayfont   = _actualdayfont.deriveFont( (float) (scale * _actualdayfont.getSize()) );
        Font datefont  = _actualdatefont.deriveFont( (float) (scale * _actualdatefont.getSize()) );
        Font diaryfont = _actualdiaryfont.deriveFont( (float) (scale * _actualdiaryfont.getSize()) );
        FontMetrics dayfontmetrics   = this.getFontMetrics( dayfont );
        FontMetrics datefontmetrics  = this.getFontMetrics( datefont );
        FontMetrics diaryfontmetrics = this.getFontMetrics( diaryfont );
        
        float dayfontheight = (float) (scale * _dayfontmetrics.getHeight());
        float dayfontascent = (float) (scale * _dayfontmetrics.getAscent());
        float datefontheight = (float) (scale * _datefontmetrics.getHeight());
        float datefontascent = (float) (scale * _datefontmetrics.getAscent());
        float diaryfontheight = (float) (scale * _diaryfontmetrics.getHeight());
        float diaryfontascent = (float) (scale * _diaryfontmetrics.getAscent());
        float diaryfontvpos = diaryfontascent + (datefontheight - diaryfontheight)/2;
        
        Rectangle2D.Float rect;

        rect = new Rectangle2D.Float( 0, 0, width, height );
        g.setColor( Color.white );
        g.fill( rect );
        
        boolean flag = false;
        float hpos = leftmargin;
        float vpos = 0;
                        
        // draw headings (day names)
        g.setFont( dayfont );
        float i2 = hpos;
        for( int col = 0; col < 7; col++ )
        {
            // draw heading text
            if( (col == 0) || (col == 6) )
                g.setColor( _cellwendtextcolor );
            else
                g.setColor( _celltextcolor );
                
            if( _useshortdaynames )
                g.drawString( Constants.shortdays[col+1], i2 + (cellwidth - dayfontmetrics.stringWidth( Constants.shortdays[col+1])) / 2, vpos + dayfontascent );
            else
                g.drawString( Constants.longdays[col+1], i2 + (cellwidth - dayfontmetrics.stringWidth( Constants.longdays[col+1])) / 2, vpos + dayfontascent );
                
            // draw cell border
            rect = new Rectangle2D.Float( i2, vpos, cellwidth, dayfontheight );
            g.setColor( _cellbordercolor );
            g.draw( rect );
            i2 += cellwidth;
        }
        
        vpos += dayfontheight;
                                
        // draw table (date numbers)
        int k1 = 1;
        flag = false;
        
        for( int row = 0; row < 6; row++ )
        {
            hpos = leftmargin;
            
            for (int col = 0; col < 7; col++)
            {
                if( row * 7 + col == _firstday )
                    flag = true;
                    
                if( flag )
                {
                    // draw cell border
                    g.setColor( _cellbordercolor );
                    rect = new Rectangle2D.Float( hpos, vpos, cellwidth, cellheight );
                    g.draw( rect );
                    
                    // draw date number    
                    if( (col == 0) || (col == 6) )
                        g.setColor( _cellwendtextcolor );
                    else
                        g.setColor( _celltextcolor );
                        
                    g.setFont( datefont );
                    g.drawString( "" + k1, hpos + 2 /*_leftmargin*/, vpos + datefontascent );
                    
                    // draw diary entries
                    if( (_diary != null) && (k1 <= 31) && (_diary[k1-1] != null) )
                    {
                        g.setColor( _celltextcolor );
                        g.setFont( diaryfont );
                        g.drawString( _diary[k1-1].getWhat(), hpos + 2 /*_leftmargin*/ + longestdate, vpos + diaryfontvpos );
                    }
                    
                    if( k1 == getDaysInMonth() )
                        flag = false;
                    
                    k1++;
                } // if flag
                hpos += cellwidth;
            } // end col
            vpos += cellheight;
        } // end row
    }
    
    ////////////////////////////////////////////////////////////////////////////
    // Accessibility stuff
    
    protected String getAccessibleName()
    {
        return resources.getString( "TableCalendar_Name" );
    }
    
    protected String getAccessibleDescription()
    {
        return resources.getString( "TableCalendar_Desc" );
    }
}