/* *****************************************************************************
 *
 * Filename: TitleCustomizer.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  18 December 2002, 17:48
 *
 * Purpose:  Customizer for all title beans.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.awt.*;
import java.beans.*;
import javax.swing.ImageIcon;
import javax.swing.JColorChooser;
import java.util.ResourceBundle;

/**
 *
 * @author  Robert
 */
public class TitleCustomizer
    extends javax.swing.JPanel
    implements java.beans.Customizer
{
    // Statics
    private static final ResourceBundle resources = ResourceBundle.getBundle("com/tsac/calmaker/beans/CalendarBeans");
    private static final int[] sizes = { 8, 10, 12, 14, 16, 18, 20, 24, 28, 32, 36 };
    
    // State
    private Title _object;
    
    // Cached
    private Image _colorIcon;
    private PropertyChangeSupport _pcsupport = null;
    
    
    /** Creates new customizer TitleCustomizer */
    public TitleCustomizer()
    {
        initComponents();
        
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        String[] fonts = ge.getAvailableFontFamilyNames();
        for( int n = 0; n < fonts.length; n ++ )
            fontFamilyCmb.addItem( fonts[n] );
        
        fontStyleCmb.addItem( resources.getString("Plain") );
        fontStyleCmb.addItem( resources.getString("Bold") );
        fontStyleCmb.addItem( resources.getString("Italic") );
        fontStyleCmb.addItem( resources.getString("Bold_Italic") );
        
        for( int n = 0; n < sizes.length; n++ )
            fontSizeCmb.addItem( Integer.toString( sizes[n] ) );
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents()//GEN-BEGIN:initComponents
    {
        java.awt.GridBagConstraints gridBagConstraints;

        textPropsPnl = new javax.swing.JPanel();
        fontLbl = new javax.swing.JLabel();
        fontPnl = new javax.swing.JPanel();
        fontFamilyCmb = new javax.swing.JComboBox();
        fontStyleCmb = new javax.swing.JComboBox();
        fontSizeCmb = new javax.swing.JComboBox();
        colorLbl = new javax.swing.JLabel();
        colorPnl = new javax.swing.JPanel();
        color = new javax.swing.JLabel();
        colorBtn = new javax.swing.JButton();

        setLayout(new java.awt.GridBagLayout());

        textPropsPnl.setLayout(new java.awt.GridBagLayout());

        fontLbl.setText(resources.getString("Font_"));
        fontLbl.setDisplayedMnemonic(resources.getString( "Font_Mnemonic" ).charAt(0));
        fontLbl.setLabelFor(fontFamilyCmb);
        fontLbl.setMinimumSize(new java.awt.Dimension(36, 16));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(8, 8, 2, 2);
        textPropsPnl.add(fontLbl, gridBagConstraints);

        fontPnl.setLayout(new java.awt.GridBagLayout());

        fontFamilyCmb.setToolTipText(resources.getString("Font_Family_Tooltip"));
        fontFamilyCmb.setMinimumSize(new java.awt.Dimension(150, 25));
        fontFamilyCmb.setPreferredSize(new java.awt.Dimension(150, 25));
        fontFamilyCmb.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                fontFamilyCmbActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 1);
        fontPnl.add(fontFamilyCmb, gridBagConstraints);

        fontStyleCmb.setToolTipText(resources.getString("Font_Style_Tooltip"));
        fontStyleCmb.setMinimumSize(new java.awt.Dimension(75, 25));
        fontStyleCmb.setPreferredSize(new java.awt.Dimension(75, 25));
        fontStyleCmb.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                fontStyleCmbActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(0, 1, 0, 1);
        fontPnl.add(fontStyleCmb, gridBagConstraints);

        fontSizeCmb.setEditable(true);
        fontSizeCmb.setToolTipText(resources.getString("Font_Size_Tooltip"));
        fontSizeCmb.setMinimumSize(new java.awt.Dimension(60, 25));
        fontSizeCmb.setPreferredSize(new java.awt.Dimension(50, 25));
        fontSizeCmb.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                fontSizeCmbActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(0, 1, 0, 0);
        fontPnl.add(fontSizeCmb, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(8, 2, 2, 8);
        textPropsPnl.add(fontPnl, gridBagConstraints);

        colorLbl.setText(resources.getString("Color_"));
        colorLbl.setDisplayedMnemonic(resources.getString( "Color_Mnemonic" ).charAt(0));
        colorLbl.setLabelFor(colorBtn);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(2, 8, 8, 2);
        textPropsPnl.add(colorLbl, gridBagConstraints);

        colorPnl.setLayout(new java.awt.GridBagLayout());

        colorPnl.setBorder(new javax.swing.border.EtchedBorder());
        color.setText("[255,255,255]");
        color.setToolTipText(resources.getString("Color_Tooltip"));
        color.setLabelFor(colorBtn);
        color.setMinimumSize(new java.awt.Dimension(110, 25));
        color.setPreferredSize(new java.awt.Dimension(110, 25));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 4, 0, 4);
        colorPnl.add(color, gridBagConstraints);

        colorBtn.setText("...");
        colorBtn.setToolTipText(resources.getString("Color_Button_Tooltip"));
        colorBtn.setMinimumSize(new java.awt.Dimension(0, 26));
        colorBtn.setPreferredSize(new java.awt.Dimension(24, 24));
        colorBtn.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                colorBtnActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        colorPnl.add(colorBtn, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 8, 8);
        textPropsPnl.add(colorPnl, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        add(textPropsPnl, gridBagConstraints);

    }//GEN-END:initComponents

    private void colorBtnActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_colorBtnActionPerformed
    {//GEN-HEADEREND:event_colorBtnActionPerformed
        JColorChooser colordlg = new JColorChooser();
        Color newColor = colordlg.showDialog( this, "Color Chooser", _object.getColor() );
        
        if( newColor != null )
        {
            Color oldColor = _object.getColor();
            
            updateColor( newColor );
            _object.setColor( newColor );
            
            if( _pcsupport != null )
                _pcsupport.firePropertyChange( "color", oldColor, newColor );            
        }
    }//GEN-LAST:event_colorBtnActionPerformed

    private void fontSizeCmbActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_fontSizeCmbActionPerformed
    {//GEN-HEADEREND:event_fontSizeCmbActionPerformed
        actionFontChanged();
    }//GEN-LAST:event_fontSizeCmbActionPerformed

    private void fontStyleCmbActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_fontStyleCmbActionPerformed
    {//GEN-HEADEREND:event_fontStyleCmbActionPerformed
        actionFontChanged();
    }//GEN-LAST:event_fontStyleCmbActionPerformed

    private void fontFamilyCmbActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_fontFamilyCmbActionPerformed
    {//GEN-HEADEREND:event_fontFamilyCmbActionPerformed
        actionFontChanged();
    }//GEN-LAST:event_fontFamilyCmbActionPerformed

    public void setObject(Object obj)
    {        
        if( obj instanceof Title )
        {
            Title t = (Title) obj;
            _object = t;
            _pcsupport = new PropertyChangeSupport( t );

            updateColor( t.getColor() );
            updateFont( t.getFont() );
        }
    }    
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton colorBtn;
    private javax.swing.JComboBox fontStyleCmb;
    private javax.swing.JPanel textPropsPnl;
    private javax.swing.JComboBox fontSizeCmb;
    private javax.swing.JLabel fontLbl;
    private javax.swing.JComboBox fontFamilyCmb;
    private javax.swing.JLabel color;
    private javax.swing.JPanel fontPnl;
    private javax.swing.JLabel colorLbl;
    private javax.swing.JPanel colorPnl;
    // End of variables declaration//GEN-END:variables

    
    public void addPropertyChangeListener( PropertyChangeListener l )
    {
        if( _pcsupport != null )
            _pcsupport.addPropertyChangeListener( l );
    }

    
    public void removePropertyChangeListener( PropertyChangeListener l )
    {
        if( _pcsupport != null )
            _pcsupport.removePropertyChangeListener( l );
    }

    
    protected void updateColor( Color c )
    {
        // Make sure the icon exists
        if( _colorIcon == null )
        {
            _colorIcon = createImage( 16, 16 );
            
            if( _colorIcon != null )
                color.setIcon( new ImageIcon( _colorIcon ) );
        }

        // Update the icon
        if( _colorIcon != null )
        {
            Graphics g = _colorIcon.getGraphics();
            g.setColor( c );
            g.fillRect( 0, 0, _colorIcon.getWidth(null), _colorIcon.getHeight(null) );
            g.setColor( Color.black );
            g.drawRect( 0, 0, _colorIcon.getWidth(null) - 1, _colorIcon.getHeight(null) - 1 );
        }
        
        // Update the text
        color.setText( "[" + c.getRed() + "," + c.getGreen() + "," + c.getBlue() + "]" );
        
        // Force a repaint
        color.repaint();
    }
    
    
    protected void updateFont( Font f )
    {
        // Save object and prevent selection events writing back to the object
        Title obj = _object;
        _object = null;
        
        fontFamilyCmb.setSelectedItem( f.getFamily() );
        fontStyleCmb.setSelectedIndex( f.getStyle() );
        fontSizeCmb.setSelectedItem( Integer.toString(f.getSize()) );
        
        // Restore the object
        _object = obj;
    }
    
    
    protected void actionFontChanged()
    {
        // Make sure object has been set
        if( _object == null )
            return;
        
        try
        {
            Font newFont = new Font( (String) fontFamilyCmb.getSelectedItem(), fontStyleCmb.getSelectedIndex(), Integer.parseInt( (String) fontSizeCmb.getSelectedItem() ) );
            Font oldFont = _object.getFont();
            
            _object.setFont( newFont );
            
            if( _pcsupport != null )
                _pcsupport.firePropertyChange( "font", oldFont, newFont );
        }
        catch( NumberFormatException e )
        {
            fontSizeCmb.setSelectedItem( Integer.toString(_object.getFont().getSize()) );
        }
    }
}
